/**********************************************************************/
/*                                                                    */
/*   Licensed Materials - Property of IBM.                            */
/*   IBM XL Fortran for Linux, V16.1.0 (5725-C75, 5765-J15)           */
/*   Copyright IBM Corp. 2003, 2018.                                  */
/*   US Government Users Restricted Rights -                          */
/*   Use, duplication or disclosure restricted by                     */
/*   GSA ADP Schedule Contract with IBM Corp.                         */
/*                                                                    */
/**********************************************************************/

#ifndef __ISO_FORTRAN_BINDING_H
#define __ISO_FORTRAN_BINDING_H

#include <sys/types.h>  /* For size_t */

typedef unsigned short CFI_attribute_t;
typedef signed short CFI_type_t;
typedef unsigned int CFI_rank_t;
typedef signed long CFI_index_t;

typedef struct
{
  CFI_index_t lower_bound;
  CFI_index_t extent;
  CFI_index_t sm;
} CFI_dim_t;

#define CFI_CDESC_T(rank_arg)                                        \
  struct                                                             \
  {                                                                  \
    /* The offset of the following members matters */                \
    void *base_addr;                                                 \
    size_t elem_len;                                                 \
    int version;                                                     \
                                                                     \
    /* The following members can be in any order */                  \
                                                                     \
    /* word boundary (32-bit mode only) */                           \
                                                                     \
    CFI_attribute_t attribute;                                       \
    CFI_type_t type;                                                 \
                                                                     \
    /* word boundary */                                              \
                                                                     \
    CFI_rank_t rank;                                                 \
                                                                     \
    /* word boundary (32-bit mode only) */                           \
                                                                     \
    int reserved1;                                                   \
    int reserved2;                                                   \
    int xlf0;                                                        \
    int xlf1;                                                        \
    int xlf2;                                                        \
    void *xlf3;                                                      \
    void *xlf4;                                                      \
    void *xlf5;                                                      \
    void *xlf6;                                                      \
    void *xlf7;                                                      \
    long xlf8;                                                       \
    void *reserved_ptr;                                              \
                                                                     \
    /* word boundary */                                              \
                                                                     \
    /* The following member must be the last member */               \
    CFI_dim_t dim[rank_arg];                                         \
  }

typedef CFI_CDESC_T() CFI_cdesc_t;

/*
 * Macros
 */

#define CFI_MAX_RANK                  (20)
#define CFI_VERSION                   (1)

/* Type attributes */
#define CFI_attribute_pointer         (1)
#define CFI_attribute_allocatable     (2)
#define CFI_attribute_other           (4)

/*
 * Type codes.
 * These should agree with the kinds in the iso_c_binding module
 */
#define CFI_type_signed_char          ( 1)
#define CFI_type_char                 CFI_type_signed_char
#define CFI_type_int8_t               ( 2)
#define CFI_type_short                ( 3)
#define CFI_type_int                  ( 4)
#define CFI_type_long                 ( 5)
#define CFI_type_long_long            ( 6)
#define CFI_type_size_t               CFI_type_long
#define CFI_type_int16_t              CFI_type_short
#define CFI_type_int32_t              CFI_type_int
#define CFI_type_int64_t              CFI_type_long_long
#define CFI_type_int_least8_t         CFI_type_int8_t
#define CFI_type_int_least16_t        CFI_type_int16_t
#define CFI_type_int_least32_t        CFI_type_int32_t
#define CFI_type_int_least64_t        CFI_type_int64_t
#define CFI_type_int_fast8_t          CFI_type_int8_t
#ifdef __linux__
#define CFI_type_int_fast16_t         CFI_type_long
#define CFI_type_int_fast32_t         CFI_type_long
#else
#define CFI_type_int_fast16_t         CFI_type_int16_t
#define CFI_type_int_fast32_t         CFI_type_int32_t
#endif
#define CFI_type_int_fast64_t         CFI_type_int64_t
#define CFI_type_intmax_t             CFI_type_int64_t
#define CFI_type_intptr_t             CFI_type_long
#define CFI_type_ptrdiff_t            CFI_type_long
#define CFI_type_float                ( 7)
#define CFI_type_double               ( 8)
#define CFI_type_long_double          ( 9)
#define CFI_type_float_Complex        (10)
#define CFI_type_double_Complex       (11)
#define CFI_type_long_double_Complex  (12)
#define CFI_type_Bool                 (13)
#define CFI_type_cptr                 (14)
#define CFI_type_struct               (15)
#define CFI_type_other                (-1)

/*
 * Error codes
 */
#define CFI_SUCCESS                   ( 0)
#define CFI_ERROR_BASE_ADDR_NULL      ( 1)
#define CFI_ERROR_BASE_ADDR_NOT_NULL  ( 2)
#define CFI_INVALID_ELEM_LEN          ( 3)
#define CFI_INVALID_RANK              ( 4)
#define CFI_INVALID_TYPE              ( 5)
#define CFI_INVALID_ATTRIBUTE         ( 6)
#define CFI_INVALID_EXTENT            ( 7)
#define CFI_INVALID_DESCRIPTOR        ( 8)
#define CFI_ERROR_MEM_ALLOCATION      ( 9)
#define CFI_ERROR_OUT_OF_BOUNDS       (10)

/*
 * Prototypes
 * The prototypes have a "__xlf_" prefix to allow coexistance
 * with object files linking to someone else's library functions.
 * (Code compiled with our compiler would go to our library,
 * code compiled with their compiler would go to their library.)
 */
void *__xlf_CFI_address(const CFI_cdesc_t *dv,
                        const CFI_index_t subscripts[]);

int __xlf_CFI_allocate(CFI_cdesc_t *dv,
                 const CFI_index_t lower_bounds[],
                 const CFI_index_t upper_bounds[],
                 size_t elem_len);

int __xlf_CFI_deallocate(CFI_cdesc_t *dv);

int __xlf_CFI_establish(CFI_cdesc_t *dv,
                        void *base_addr,
                        CFI_attribute_t attribute,
                        CFI_type_t type,
                        size_t elem_len,
                        CFI_rank_t rank,
                        const CFI_index_t extents[]);

int __xlf_CFI_is_contiguous(const CFI_cdesc_t *dv);

int __xlf_CFI_section(CFI_cdesc_t *result,
                      const CFI_cdesc_t *source,
                      const CFI_index_t lower_bounds[],
                      const CFI_index_t upper_bounds[],
                      const CFI_index_t strides[]);

int __xlf_CFI_select_part(CFI_cdesc_t *result,
                          const CFI_cdesc_t *source,
                          size_t displacement,
                          size_t elem_len);

int __xlf_CFI_setpointer(CFI_cdesc_t *result,
                         CFI_cdesc_t *source,
                         const CFI_index_t lower_bounds[]);

/*
 * IBM extension function prototypes
 */
char *__xlf_CFI_strerror(int ind);

/*
 * Macros to map the standard names to our prototypes
 */
#define CFI_address                   __xlf_CFI_address
#define CFI_allocate                  __xlf_CFI_allocate
#define CFI_deallocate                __xlf_CFI_deallocate
#define CFI_establish                 __xlf_CFI_establish
#define CFI_is_contiguous             __xlf_CFI_is_contiguous
#define CFI_section                   __xlf_CFI_section
#define CFI_select_part               __xlf_CFI_select_part
#define CFI_setpointer                __xlf_CFI_setpointer


#endif /* __ISO_FORTRAN_BINDING_H */
