/**********************************************************************/
/*                                                                    */
/*   Licensed Materials - Property of IBM.                            */
/*   IBM XL Fortran for Linux, V16.1.0 (5725-C75, 5765-J15)           */
/*   Copyright IBM Corp. 2003, 2018.                                  */
/*   US Government Users Restricted Rights -                          */
/*   Use, duplication or disclosure restricted by                     */
/*   GSA ADP Schedule Contract with IBM Corp.                         */
/*                                                                    */
/**********************************************************************/

#include <stdio.h>

/*
 * Simple tracing functions for use with -qfunctrace
 *
 * These functions are written in C to avoid Fortran I/O recusion
 * when tracing foo() in the following case:   print *, foo()
 *
 * To use these functions:
 *   cc -c tracing_routines.c
 *   xlf yourfile.f -qfunctrace tracing_routines.o
 */

void
__func_trace_enter(const char * const proc_name,
                   const char * const file_name,
                   const int line_no,
                   void ** const id)
{
  printf("{ %s (%s:%d)\n", proc_name, file_name, line_no);
  fflush(stdout);
}

void
__func_trace_exit(const char * const proc_name,
                  const char * const file_name,
                  const int line_no,
                  void ** const id)
{
  printf("} %s (%s:%d)\n", proc_name, file_name, line_no);
  fflush(stdout);
}

void
__func_trace_catch(const char * const proc_name,
                   const char * const file_name,
                   int line_no,
                   void ** const id)
{
  printf("  %s (%s:%d) catch block\n", proc_name, file_name, line_no);
  fflush(stdout);
}
