!***********************************************************************
!*   
!*   Licensed Materials - Property of IBM.
!*   IBM XL Fortran for Linux, V16.1.0 (5725-C75, 5765-J15)
!*   Copyright IBM Corp. 2003, 2018.
!*   US Government Users Restricted Rights - 
!*   Use, duplication or disclosure restricted by 
!*   GSA ADP Schedule Contract with IBM Corp.
!*
!***********************************************************************

!
! Simple tracing procedures for use with -qfunctrace
!
! These subroutines demonstrate how to write tracing routines
! in Fortran.  They must not be used to trace functions that
! appear in Fortran I/O lists since that will cause invalid
! Fortran I/O recursion.  For example, do not use these
! subroutines to trace foo in the following case:
! print *, foo()
!
! To use these subroutines:
!   xlf -c tracing_routines.f90
!   xlf yourfile.f -qfunctrace tracing_routines.o
!

!IBM* SOURCEFORM(FREE(F90))
!IBM* FUNCTRACE_XLF_ENTER
subroutine my__func_trace_enter(proc_name, file_name, line_no, id)
    use, intrinsic :: iso_c_binding
    character(*), intent(in) :: proc_name
    character(*), intent(in) :: file_name
    integer(c_int), intent(in) :: line_no
    type(c_ptr), intent(inout) :: id

    print '("{ ",A," (",A,":",I0,")")', proc_name, file_name, line_no
end subroutine

!IBM* SOURCEFORM(FREE(F90))
!IBM* FUNCTRACE_XLF_EXIT
subroutine my__func_trace_exit(proc_name, file_name, line_no, id)
    use, intrinsic :: iso_c_binding
    character(*), intent(in) :: proc_name
    character(*), intent(in) :: file_name
    integer(c_int), intent(in) :: line_no
    type(c_ptr), intent(inout) :: id

    print '("} ",A," (",A,":",I0,")")', proc_name, file_name, line_no
end subroutine

!IBM* SOURCEFORM(FREE(F90))
!IBM* FUNCTRACE_XLF_CATCH
subroutine my__func_trace_catch(proc_name, file_name, line_no, id)
    use, intrinsic :: iso_c_binding
    character(*), intent(in) :: proc_name
    character(*), intent(in) :: file_name
    integer(c_int), intent(in) :: line_no
    type(c_ptr), intent(inout) :: id

    print '("  ",A," (",A,":",I0,") catch block")', &
          proc_name, file_name, line_no
end subroutine
